#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "dockets.h"
#include "pizza_utils.h"
#include "order_entry.h"

static DataEntry *display_docket_screen = NULL;
static char *the_display_docket = NULL;
static GtkText *the_text;

static int docket_store_id, docket_order_id;

int order_came_from_docket = FALSE;
static int allow_edits = TRUE;

/*----------------------------------------------------------
::docket routines
-----------------------------------------------------------*/

enum {
	DISPLAY_DOCKET_TEXT,

	DISPLAY_DOCKET_SA_BTN,
	DISPLAY_DOCKET_RP_BTN,
	DISPLAY_DOCKET_EDIT_BTN,
	DISPLAY_DOCKET_QUIT_BTN
};

void display_docket_quit ()
{
	gtk_widget_hide (GTK_WIDGET(display_docket_screen));
}

void display_docket_sa ()
{
	char command[2048];
	int new_order;

	if (!allow_edits) return;

	// get a new order number, and create a new header

	new_order = get_new_order_number ();
	res = execSQL ("insert into orders (ordernum,store_id,is_pickup,custnum,order_date,"
			"total_price,comments) "
			"select %d,%d,'f',custnum,'now',total_price,comments from orders "
			"where store_id = %d and ordernum = %d",
			new_order, current_store,
			docket_store_id, docket_order_id);
	PQclear (res);

	// add the order details
	res = execSQL ("insert into ordet (ordernum,store_id,seq_num,item_code,description,qty,price) "
		"select %d,%d,seq_num,item_code,description,qty,price from ordet "
		"where store_id = %d and ordernum = %d",
		new_order, current_store,
		docket_store_id, docket_order_id);
	PQclear (res);

	order_came_from_docket = TRUE;
	gtk_widget_hide (GTK_WIDGET(display_docket_screen));
	old_order (current_store, new_order,FALSE);
}

void display_docket_rp ()
{
	char filename[32];
	char command[80];
	FILE *fp;

	sprintf (filename, "%d-%d.docket", docket_store_id, docket_order_id);
	fp = fopen (filename, "w");
	if (!fp)
	{
		sprintf (command, "Cannot open docket file %s\n\n- No docket will be printed",
			filename);
		message_box (command);
		return;
	}
	fprintf (fp, "%s", the_display_docket);
	fclose (fp);

	sprintf (command, "print_docket %s", filename);
	system (command);
}

void display_docket_edit ()
{
	if (!allow_edits) return;

	order_came_from_docket = TRUE;
	gtk_widget_hide (GTK_WIDGET(display_docket_screen));
	old_order (docket_store_id, docket_order_id, FALSE);
}

void hide_docket_display ()
{
	if (display_docket_screen)
		gtk_widget_hide (GTK_WIDGET(display_docket_screen));

	the_display_docket = NULL;
}

void display_docket (int store_id, int order_id, int allow_order)
{
	static GtkWidget *sa_btn, *edit_btn;

	docket_store_id = store_id;
	docket_order_id = order_id;

	the_display_docket = generate_docket (store_id, order_id);
	allow_edits = allow_order;

	if (!display_docket_screen)
	{
		display_docket_screen = DATA_ENTRY(data_entry_new ("Docket", 400, 100, 0,0));

		gtk_widget_realize (GTK_WIDGET(display_docket_screen));

		the_text = GTK_TEXT(gtk_text_new (NULL,NULL));
		gtk_widget_set_usize (GTK_WIDGET(the_text), 360,360);
		gtk_text_set_editable (the_text, TRUE);

		data_entry_add_widget (display_docket_screen, DISPLAY_DOCKET_TEXT,
			GTK_WIDGET(the_text), 0,0,1,1);

		sa_btn = data_entry_add_label_button (display_docket_screen, DISPLAY_DOCKET_SA_BTN, "Same Again", 
				GTK_SIGNAL_FUNC(display_docket_sa), BUTTON_NORMAL);
		data_entry_add_label_button (display_docket_screen, DISPLAY_DOCKET_RP_BTN, "Re Print", 
				GTK_SIGNAL_FUNC(display_docket_rp), BUTTON_NORMAL);
		edit_btn = data_entry_add_label_button (display_docket_screen, DISPLAY_DOCKET_EDIT_BTN, "Edit", 
				GTK_SIGNAL_FUNC(display_docket_edit), BUTTON_NORMAL);
		data_entry_add_label_button (display_docket_screen, DISPLAY_DOCKET_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(display_docket_quit), BUTTON_QUIT);

		gtk_widget_realize (GTK_WIDGET(the_text));
	}

	gtk_editable_delete_text (GTK_EDITABLE(the_text),0,-1);
	gtk_text_insert (the_text, NULL,NULL,NULL, the_display_docket, strlen(the_display_docket));
	if (!GTK_WIDGET_VISIBLE(display_docket_screen))
	{
		gtk_widget_show (GTK_WIDGET(display_docket_screen));
	}
	gtk_widget_grab_focus (GTK_WIDGET(the_text));

	if (allow_edits)
	{
		gtk_widget_show (sa_btn);
		gtk_widget_show (edit_btn);
	}
	else
	{
		gtk_widget_hide (sa_btn);
		gtk_widget_hide (edit_btn);
	}

	gdk_window_raise (GTK_WIDGET(display_docket_screen)->window);
}

void print_docket (int store_id, int order_id)
{
	char filename[32];
	char command[80];
	FILE *fp;
	char *generated_docket;
	
	generated_docket = generate_docket (store_id, order_id);

	sprintf (filename, "%d-%d.docket", store_id, order_id);
	fp = fopen (filename, "w");
	if (!fp)
	{
		sprintf (command, "Cannot open docket file %s\n\n- No docket will be printed",
			filename);
		message_box (command);
		return;
	}
	fprintf (fp, "%s", generated_docket);
	fclose (fp);

	sprintf (command, "print_docket %s", filename);
	system (command);
}

void add_comment_string (char *str, char *ptr)
{
	char *ptr1;
	int col;

	trim (ptr);
	if (!strlen(ptr)) return;

	ptr1 = str + strlen (str);
	col = 0;

	*ptr1++ = '\n';
	*ptr1 = 0;

	while (1)
	{
		switch (*ptr)
		{
			case 0:
				*ptr1++ = '\n';
				*ptr1++ = '\n';
				*ptr1 = 0;
				return;

			case '\n':
				*ptr1++ = *ptr++;
				col = 0;
				break;

			default:
				*ptr1++ = *ptr++;
				col++;
				*ptr1 = 0;

				if (col >= 40)
				{
					*ptr1++ = '\n';
					*ptr1 = 0;
					col = 0;
				}
				break;
		}
	}
}

char *make_desc_string (char *ptr)
{
	static char result[1024];

	sprintf (result, "%-29.29s", ptr);

	while (strlen (ptr) > 29)
	{
		ptr += 29;
		sprintf (result + strlen (result), "\n     %-29.29s", ptr);
	}

	return result;
}

char *generate_docket (int store_id, int order_id)
{
	int i,nTuples;
	PGresult *dres,*cres,*ores;
	char command[256];
	static char result[8192];
	int custnum;
	int is_pickup;
	int theQty;
	char *desc_string;

	memset (result, 0, sizeof (result));

	dres = execSQL ("select is_pickup,custnum,pickup_name,pickup_time,order_date,total_price,comments"
			" from orders where store_id = %d and ordernum = %d",
			store_id, order_id);
	if (!dres) return result;

	is_pickup = *PQgetvalue (dres,0,0) == 't';

	if (is_pickup)
	{
		sprintf (result, 	"Order: %d-%d\n"
							"Date:  %s\n\n"
							"Pickup: %s\n"
							"At:%s\n",
				store_id, order_id, 
				nice_date (PQgetvalue (dres,0,4)),
				PQgetvalue (dres,0,2),
				PQgetvalue (dres,0,3)
			);
		add_comment_string (result + strlen(result), PQgetvalue (dres,0,6));
		sprintf (result + strlen (result),
							"Description                          Price\n"
							"==========================================\n");

		ores = execSQL ("select qty,seq_num,ordet.description,ordet.price,print_qty "
						  "from ordet,items where store_id = %d and ordernum = %d "
						  "and code = item_code order by seq_num",
						  store_id, order_id);
		nTuples = PQntuples (ores);
		for (i = 0; i < nTuples; i++)
		{
			desc_string = make_desc_string (PQgetvalue (ores,i,2));
			if (*PQgetvalue (ores,i,4) == 't')
			{
				sprintf (result + strlen(result),
							"%2d x %s$%s\n",
							atoi(PQgetvalue (ores,i,0)),
							desc_string,
							money (atof(PQgetvalue (ores,i,3)), 7)
						);
			}
			else
			{
				sprintf (result + strlen(result),
							"     %s$%s\n",
							desc_string,
							money (atof(PQgetvalue (ores,i,3)), 7)
						);
			}
		}
		PQclear (ores);

		sprintf (result + strlen(result), "\n"
							"                                   -------\n"
                            "                          Total   $%s\n\n\n"
							"      Enjoy your meal !\n\n\n\n\n\n\n\n",
				money (atof (PQgetvalue (dres,0,5)), 7)
			);
		PQclear (dres);
		return result;
	}
	else
	{
		custnum = atoi(PQgetvalue (dres,0,1));
		if (!custnum) 
		{
			PQclear (dres);
			return result;
		}

		cres = execSQL ("select custname,address,suburb,phone from customers "
					"where custnum = %d",
					custnum);

		sprintf (result, 	"Order: %d-%d\n"
							"Date:  %s\n\n"
							"Name:  %s\n"
							"Addr:  %s\n"
							"       %s\n"
							"Phone: %s\n\n",
					store_id, order_id,
					nice_date(PQgetvalue (dres,0,4)),
					PQgetvalue (cres,0,0),
					PQgetvalue (cres,0,1),
					PQgetvalue (cres,0,2),
					PQgetvalue (cres,0,3)
				);

		add_comment_string (result + strlen(result), PQgetvalue (dres,0,6));
		sprintf (result + strlen (result),
							"Description                          Price\n"
							"==========================================\n");

		ores = execSQL ("select qty,seq_num,ordet.description,ordet.price,print_qty "
						  "from ordet,items where store_id = %d and ordernum = %d "
						  "and code = item_code order by seq_num",
						  store_id, order_id);
		nTuples = PQntuples (ores);

		for (i = 0; i < nTuples; i++)
		{
			desc_string = make_desc_string (PQgetvalue (ores,i,2));
			theQty = atoi(PQgetvalue (ores,i,0));

			if (*PQgetvalue (ores,i,4) == 't')
			{
				if (theQty)
					sprintf (result + strlen(result), "%2d x ", theQty);
				else
					strcat (result, "     ");
				sprintf (result + strlen(result),
							"%s$%s\n",
							desc_string,
							money (atof(PQgetvalue (ores,i,3)), 7)
						);
			}
			else
			{
				sprintf (result + strlen(result),
							"     %s$%s\n",
							desc_string,
							money (atof(PQgetvalue (ores,i,3)), 7)
						);
			}
		}
		PQclear (ores);

		sprintf (result + strlen(result), 
							"                                   -------\n"
                            "                          Total   $%s\n\n\n"
							"      Enjoy your meal !\n\n\n\n\n\n\n\n",
					money (atof (PQgetvalue (dres,0,5)), 7)
				);

		PQclear (dres);
		PQclear (cres);

		return (result);
	}


	return result;
}
