#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include <gdk_imlib.h>
#include <gdk/gdk.h>
#include "pizza_utils.h"

#include "cust_maint.h"
#include "suburb_maint.h"
#include "item_maint.h"
#include "store_maint.h"
#include "user_maint.h"
#include "cat_maint.h"
#include "select_screen.h"

#include "reports.h"

DataEntry *login_screen;

/*----------------------------------------------------------------------------------------------------
::maintenance toolbar
----------------------------------------------------------------------------------------------------*/
enum {
	MAINT_TOOLBAR,

	MAINT_QUIT_BTN
};

void maint_callback (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	int btn_num;

	data_entry = (DataEntry *) data;
	btn_num = data_entry_get_widget_id (data_entry, widget);

	switch (btn_num)
	{
		case MAINT_QUIT_BTN:
			gtk_widget_show (GTK_WIDGET(login_screen));
			gtk_widget_hide (GTK_WIDGET(data_entry));

			if (maint_store_data_entry) gtk_widget_hide (GTK_WIDGET(maint_store_data_entry));
			if (maint_user_data_entry) gtk_widget_hide (GTK_WIDGET(maint_user_data_entry));
			if (maint_item_data_entry) gtk_widget_hide (GTK_WIDGET(maint_item_data_entry));
			if (maint_cat_data_entry) gtk_widget_hide (GTK_WIDGET(maint_cat_data_entry));
			if (maint_suburb_data_entry) gtk_widget_hide (GTK_WIDGET(maint_suburb_data_entry));
			if (maint_cust_data_entry) gtk_widget_hide (GTK_WIDGET(maint_cust_data_entry));
			if (report_selection) gtk_widget_hide (GTK_WIDGET(report_selection));
			break;
	}
}

static void do_backup ()
{
	system ("backup");
}

void do_maint_menu ()
{
	static DataEntry *data_entry = NULL;
	GtkWidget *window;
	GtkWidget *toolbar;
	// GtkEntry *desc;

	if (!data_entry)
	{

	data_entry = DATA_ENTRY(data_entry_new ("Maintenance", 20, 20, 0,0));
	window = (GtkWidget *) data_entry;
	gtk_widget_realize (window);
	
	/*--------- Add a toolbar ----------*/

	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);

	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar),
		"Cat", "Item Categories Maintenance", "Toolbar/Cat",
		new_pixmap ("file.xpm", window->window, &window->style->bg[GTK_STATE_NORMAL]),
		(GtkSignalFunc) maint_cats, data_entry);

	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar),
		"Stores", "Store Maintenance", "Toolbar/Stores",
		new_pixmap ("file.xpm", window->window, &window->style->bg[GTK_STATE_NORMAL]),
		(GtkSignalFunc) maint_stores, data_entry);

	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar),
		"Users", "User Maintenance", "Toolbar/Users",
		new_pixmap ("file.xpm", window->window, &window->style->bg[GTK_STATE_NORMAL]),
		(GtkSignalFunc) maint_users, data_entry);

	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar),
		"Items", "Items Maintenance", "Toolbar/Items",
		new_pixmap ("file.xpm", window->window, &window->style->bg[GTK_STATE_NORMAL]),
		(GtkSignalFunc) maint_items, data_entry);

	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar),
		"Suburbs", "Suburb Maintenance", "Toolbar/Suburbs",
		new_pixmap ("file.xpm", window->window, &window->style->bg[GTK_STATE_NORMAL]),
		(GtkSignalFunc) maint_suburbs, data_entry);

	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar),
		"Customers", "Customer Maintenance", "Toolbar/Customers",
		new_pixmap ("file.xpm", window->window, &window->style->bg[GTK_STATE_NORMAL]),
		(GtkSignalFunc) maint_custs, data_entry);

	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar),
		"Reports", "Reports", "Toolbar/SalesReport",
		new_pixmap ("report.xpm", window->window, &window->style->bg[GTK_STATE_NORMAL]),
		(GtkSignalFunc) do_reports, data_entry);

	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar),
		"Changes", "Changed Orders Report", "Toolbar/Changed",
		new_pixmap ("report.xpm", window->window, &window->style->bg[GTK_STATE_NORMAL]),
		(GtkSignalFunc) changed_orders_report, data_entry);

	gtk_toolbar_append_item (GTK_TOOLBAR(toolbar),
		"Backup", "Backup Files", "Toolbar/Backup",
		new_pixmap ("file.xpm", window->window, &window->style->bg[GTK_STATE_NORMAL]),
		(GtkSignalFunc) do_backup, data_entry);

	data_entry_add_widget (data_entry, MAINT_TOOLBAR, toolbar, 0,0,1,1);

	/*--------- End Add a toolbar ------*/

	data_entry_add_label_button (data_entry, MAINT_QUIT_BTN, "Quit", 
		GTK_SIGNAL_FUNC(maint_callback), BUTTON_QUIT);

	}

	if (!GTK_WIDGET_VISIBLE (data_entry))
		gtk_widget_show (GTK_WIDGET(data_entry));
}

/*----------------------------------------------------------------------------------------------------
::login_callback
----------------------------------------------------------------------------------------------------*/

enum {
	LOGIN_PIX,
	LOGIN_NAME_FIELD,
	LOGIN_PASSWD_FIELD,

	LOGIN_BTN,
	LOGIN_QUIT_BTN
};

void login_callback (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	int btn_num;
	GtkInput *name, *passwd;
	char *name_text, *passwd_text;

	char command[255];

	data_entry = (DataEntry *) data;
	btn_num = data_entry_get_widget_id (data_entry, widget);

	switch (btn_num)
	{
		case LOGIN_BTN: // login
			//printf ("login:\n");
			name = GTK_INPUT(data_entry_get_widget (data_entry, LOGIN_NAME_FIELD));
			passwd = GTK_INPUT(data_entry_get_widget (data_entry, LOGIN_PASSWD_FIELD));
			name_text = (char *) gtk_input_get_text (name);
			passwd_text = (char *) gtk_input_get_text (passwd);
			//printf ("\tName: %s\n", name_text);
			//printf ("\tPasswd: %s\n", passwd_text);

	if (!strcmp (name_text, "MAINT") && !strcmp (passwd_text, "dog3"))
	{
		gtk_input_set_text (name, "");
		gtk_input_set_text (passwd, "");
		gtk_widget_grab_focus (GTK_WIDGET(name));
		gtk_widget_hide (GTK_WIDGET(data_entry));
		do_maint_menu ();
		return;
	}

	res = execSQL ("select * from users where username = '%s'", name_text);
    if (PQresultStatus(res) != PGRES_TUPLES_OK)
    {
        fprintf(stderr, "'%s' didn't return tuples properly\n", command);
        PQclear(res);
        exit_nicely(conn);
    }

	if (PQntuples(res) && !strcmp (passwd_text, PQgetvalue (res, 0, 1)))
	{
		//printf ("Passed\n");
		gtk_widget_hide (GTK_WIDGET(data_entry));
		PQclear (res);
		do_select_screen ();
	}
	else
	{
		printf ("Login Failed\n");
    	PQclear(res);
		//printf ("Was '%s'\n", PQgetvalue (res, 0,1));
	}
		gtk_input_set_text (name, "");
		gtk_input_set_text (passwd, "");
		gtk_widget_grab_focus (GTK_WIDGET(name));

  
			break;

		case LOGIN_QUIT_BTN: // quit
			gtk_main_quit ();
			break;
	}
}

/*----------------------------------------------------------------------------------------------------
::do_login

create and show the login window, validate the login or quit
----------------------------------------------------------------------------------------------------*/

void do_login (gchar *title)
{
	GtkInput *name;
	GtkInput *password;

	login_screen = DATA_ENTRY(data_entry_new (title, 20, 20, 0,0));

	data_entry_add_image (login_screen, LOGIN_PIX, "Pizza.jpg",
		0,0,3,1);
	name = GTK_INPUT(data_entry_add_field (login_screen, LOGIN_NAME_FIELD, 
		"Name:", 16, INPUT_UPPER, 1,1,1,1));
	password = GTK_INPUT(data_entry_add_field (login_screen, LOGIN_PASSWD_FIELD, 
		"Password:", 16, INPUT_NORMAL, 1,2,1,1));
		password->echo_char = '*';

	data_entry_add_label_button (login_screen, LOGIN_BTN, "Login", 
		GTK_SIGNAL_FUNC(login_callback), BUTTON_DEFAULT);
	data_entry_add_label_button (login_screen, LOGIN_QUIT_BTN, "Quit", 
		GTK_SIGNAL_FUNC(login_callback), BUTTON_NORMAL);

	gtk_widget_grab_focus (GTK_WIDGET(name));
	gtk_widget_show (GTK_WIDGET(login_screen));
}

/*----------------------------------------------------------------------------------------------------
::main

main routine
----------------------------------------------------------------------------------------------------*/

static void usage ()
{
	fprintf (stderr, "USAGE: gtk_pizza [-s store_id] [-db database] [-h postgres_host] [-u userid]\n");
	fprintf (stderr, "   store_id = the store id for this store, default = 1\n");
	fprintf (stderr, "   database = the database name to connect to, default = hotpizza\n");
	fprintf (stderr, "   postgres_host = the database host computer, default = NULL\n");
	fprintf (stderr, "   user = the database userid (ie 'postgres' = 500), default = 500\n");
	exit (1);
}

int main (int argc, char **argv)
{
	FILE *debug_file;
	char *hostname = NULL;
	char *databasename = "hotpizza";
	int  userid = 500;
	int i;
	char WindowTitle[32];

	char *image_cache[] = {
		"Menu1.jpg",
		"Menu2.jpg",
		"Menu3.jpg",
		"Menu4.jpg",
		"Menu5.jpg",
		"Menu6.jpg",
		"Menu7.jpg",
		"Menu8.jpg",
		"Menu9.jpg",
		"Menu10.jpg",
		"Menu_Keys.jpg",
		"Select.jpg",
		"Pizza.jpg",
		"NewCust.jpg","Customer.jpg","BadCust.jpg",
		NULL
	};

	system ("clear_printer");

	// init gtk & imlib

	gtk_set_locale ();
	gtk_init (&argc, &argv);
	gdk_imlib_init ();

	gtk_widget_push_visual (gdk_imlib_get_visual());
	gtk_widget_push_colormap (gdk_imlib_get_colormap());

	gtk_rc_parse ("gtk_pizza.rc");

	// parse cmdline

	current_store = 1;

	for (i = 1; i < argc; i++)
	{
		if (!strcmp (argv[i], "-s"))
		{
			i++;
			if (i >= argc) usage ();
			current_store = atoi (argv[i]);
		}
		else if (!strcmp (argv[i], "-h"))
		{
			i++;
			if (i >= argc) usage ();
			hostname = argv[i];
		}
		else if (!strcmp (argv[i], "-u"))
		{
			i++;
			if (i >= argc) usage ();
			userid = atoi(argv[i]);
		}
		else if (!strcmp (argv[i], "-db"))
		{
			i++;
			if (i >= argc) usage ();
			databasename = argv[i];
		}
		else
		{
			fprintf (stderr, "Unknown option '%s'\n\n", argv[i]);
			usage ();
		}
	}

	sprintf (WindowTitle, "Login Store %d", current_store);

	// connect to postgres

	seteuid (userid);

	pghost = hostname;
    pgport = NULL;              /* port of the backend server */
    pgoptions = NULL;           /* special options to start up the backend
                                 * server */
    pgtty = NULL;               /* debugging tty for the backend server */

    /* make a connection to the database */
    conn = PQsetdb(pghost, pgport, pgoptions, pgtty, databasename);

    /* check to see that the backend connection was successfully made */
    if (PQstatus(conn) == CONNECTION_BAD)
    {
        fprintf(stderr, "Connection to database '%s' failed.\n", databasename);
        fprintf(stderr, "%s", PQerrorMessage(conn));
        exit_nicely(conn);
    }
	seteuid (0);
	//debug_file = fopen ("debug.sql", "w");
	//PQtrace (conn, debug_file);
 
	/* start gtk */

	do_login (WindowTitle);

	//cache_images (image_cache);

	gtk_main ();
	return 0;
}

/*----------------------------------------------------------------------------------------------------
::function_name
----------------------------------------------------------------------------------------------------*/
