//#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"
#include "select_screen.h"
#include "cust_details.h"
#include "dockets.h"
#include "order_entry.h"
#include "inform_all.h"
#include "display_file.h"

static void calc_total_price ();
static void set_total_price ();
static void default_focus ();

static DataEntry *order_entry_data_entry = NULL;

static GtkPixmap *pix,*pixmenu;

static char *current_pix;

static int the_order, the_store, the_customer;
static int do_change_code;

static GtkLabel *cust_label, *deliv_label;
static GtkInput *pick_name, *pick_time;
static int is_new, is_pickup;
static GtkButton *total_btn;
static GtkLabel *total_price;
static double total_price_value = 0.0;

static GtkLabel *l1, *l2;
static GtkText *comments;

static float apply_discount;
static float cust_deliv_charge = 0.0;

// NUM ROWSS is the number of off-screen rows to hold. Must be an integral
// multiple of NUM_ROWS

#define NUM_ROWS 15
#define NUM_ROWSS 150

static GtkInput *code[NUM_ROWS], *desc[NUM_ROWS], *qty[NUM_ROWS], *price[NUM_ROWS];
static GtkPixmap *bubble[NUM_ROWS];

struct row_data_struct {
	char code[10];
	char desc[256];
	int qty;
	float price;
	int change_price;
	float unit_price;
	int allow_discount;
} row_data [NUM_ROWSS];

static int display_page = 0;

#define ABS_ROW(i) (i + (display_page * NUM_ROWS))

/*----------------------------------------------------------------------------------------------------
::row_data functions
----------------------------------------------------------------------------------------------------*/

static void clear_bubble ()
{
	reload_image (bubble [display_page], "sphere1.jpg");
}

static void show_bubble ()
{
	reload_image (bubble [display_page], "sphere2.jpg");
}

static void zap_row_data ()
{
	memset (row_data, 0, sizeof (row_data));
}

static void row_data_to_screen ()
{
	int i, j;
	char *ptr;
	static char buffer[32];
	int old_change = do_change_code;
	do_change_code = 0;

	for (i = 0; i < NUM_ROWS; i++)
	{
		j = ABS_ROW(i);
		ptr = row_data[j].code;
		gtk_input_set_text (code[i], ptr ? ptr : "");
		ptr = row_data[j].desc;
		gtk_input_set_text (desc[i], ptr ? ptr : "");
		sprintf (buffer, "%d", row_data[j].qty);
		gtk_input_set_text (qty[i], row_data[j].qty ? buffer : "");
		sprintf (buffer, "%.02f", row_data[j].price);
		if (row_data[j].price == 0.0)
			gtk_input_set_text (price[i], "");
		else
			gtk_input_set_text (price[i], money (row_data[j].price, MLEN));

		gtk_input_set_editable (price[i], row_data[j].change_price);
	}

	do_change_code = old_change;
}

static void row_to_row_data (int i)
{
	int j;
	char *ptr;

	j = ABS_ROW(i);
	//memset (row_data[j], 0, sizeof (struct row_data_struct));

	ptr = gtk_input_get_text (code[i]);
	strcpy (row_data[j].code, ptr);
	ptr = gtk_input_get_text (desc[i]);
	strcpy (row_data[j].desc, ptr);
	row_data[j].qty = atoi (gtk_input_get_text (qty[i]));
	row_data[j].price = atof (gtk_input_get_text (price[i]));

	calc_total_price ();
}

static void screen_to_row_data ()
{
	int i;

	for (i = 0; i < NUM_ROWS; i++)
	{
		row_to_row_data (i);
	}
}

static float discounted (float f)
{
	return f * apply_discount;
}

/*----------------------------------------------------------------------------------------------------
::order_entry functions
----------------------------------------------------------------------------------------------------*/

enum {
	ORDER_ENTRY_HISTORY_BTN = NUM_ROWS * 4,
	ORDER_ENTRY_ACCEPT_BTN,
	ORDER_ENTRY_ADP_BTN,
	ORDER_ENTRY_CANCEL_BTN,
	ORDER_ENTRY_QUIT_BTN,

	ORDER_ENTRY_PICKUP_NAME,
	ORDER_ENTRY_PICKUP_TIME
};

/*--------------------------------------
   some routines to handle order history
----------------------------------------*/

static int check_if_has_history ()
{
	char command[255];
	int retval = 0;

	if (is_new)
		return retval;

	// generate a history report file
	res = execSQL ("select count(*) from ordhist where store_id = %d and ordernum = %d",
		the_store, the_order);
	retval = atoi(PQgetvalue (res,0,0));
	PQclear (res);

	return retval;
}

static void order_entry_history_cb ()
{
	int i, nTuples;
	int j, nTuples2;
	char command[255];
	FILE *fp;
	PGresult *res2;
	float v,total;

	if (is_new)
		return;

	fp = fopen ("order.history", "w");
	if (!fp) return;

	// generate a history report file
	res = execSQL ("select distinct date from ordhist where store_id = %d and ordernum = %d "
			"order by date DESC",
		the_store, the_order);
	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		total = 0;
		fprintf (fp, "Date: %24.24s\n\n", PQgetvalue (res,i,0));
		res2 = execSQL ("select item_code,description,qty,price,seq_num from ordhist "
						"where ordernum = %d and store_id = %d and date = '%24.24s' "
						"order by seq_num",
						the_order, the_store, PQgetvalue (res,i,0));
		nTuples2 = PQntuples (res2);

		for (j = 0; j < nTuples2; j++)
		{
			v = atof (PQgetvalue (res2,j,3));
			total += v;
			fprintf (fp, "%-5.5s %-50.50s %4.4s %s\n",
				PQgetvalue (res2, j, 0),
				PQgetvalue (res2, j, 1),
				PQgetvalue (res2, j, 2),
				money (v,MLEN)
				);
		}

		PQclear (res2);
		fprintf (fp, "\n                                                       Total: %s\n",
			money (total, 7));
		fprintf (fp, "----------------------------------------------------------------------\n\n");
	}
	PQclear (res);
	fclose (fp);

	display_file ("order.history");
	gtk_widget_grab_focus (GTK_WIDGET(code[0]));
}

/*--------------------------------------
   end of history routines
----------------------------------------*/


static void order_entry_quit_cb ()
{
	hide_file_display ();
	gtk_widget_hide (GTK_WIDGET (order_entry_data_entry));

	if (!order_came_from_docket)
		select_screen_redisplay ();
	else
		refresh_cust_details ();
		
	order_came_from_docket = FALSE;
}

static void order_entry_adp_cb ()
{
	char command[1024];
	int i,j;

	if (is_new)
	{
		// insert a new order 

		the_order = get_new_order_number ();
		res = execSQL ("insert into orders (ordernum,store_id,is_pickup,custnum,"
				"pickup_name,pickup_time,order_date,total_price,comments) "
				"values (%d,%d,'%c',%d,'%s','%s','now',%.02f,'%s')",
				the_order,
				current_store,
				is_pickup ? 't' : 'f',
				the_customer,
				SQL(gtk_input_get_text (pick_name)),
				SQL(gtk_input_get_text (pick_time)),
				total_price_value,
				SQL(gtk_editable_get_chars (comments,0,-1))
				);
			
		PQclear (res);
		inform_all ();
	}
	else if (the_store == current_store)
	{
		// update the existing order record
		res = execSQL ("update orders set pickup_name = '%s', pickup_time = '%s',"
				"total_price = %.02f, comments = '%s' where store_id = %d and ordernum = %d",
				SQL(gtk_input_get_text (pick_name)),
				SQL(gtk_input_get_text (pick_time)),
				total_price_value,
				SQL(gtk_editable_get_chars (comments,0,-1)),
				the_store,
				the_order);
			
		PQclear (res);

		// copy the existing ordet to the audit trail
		res = execSQL ("insert into ordhist (ordernum,seq_num,store_id,item_code,"
						"description,qty,price,date) "
						"select ordernum,seq_num,store_id,item_code,"
						"description,qty,price,'now' from ordet "
						"where store_id = %d and ordernum = %d",
						the_store,
						the_order);  

		PQclear (res);

		// delete the old ordet
		res = execSQL ("delete from ordet where store_id = %d and ordernum = %d",
						the_store, the_order);  
		PQclear (res);
		inform_all ();
	}

	if (the_store == current_store)
	{
		// insert the new ordets, if we are in the same store
		j = 1;

		for (i = 0; i < NUM_ROWSS; i++)
		{
			if (strlen (row_data[i].code))
			{
				res = execSQL ("insert into ordet (ordernum,store_id,seq_num,"
								"item_code,description,qty,price) values "
								"(%d,%d,%d,'%s','%s',%d,%.02f)",
								the_order, the_store, j++,
								SQL(row_data[i].code),
								SQL(row_data[i].desc),
								row_data[i].qty,
								row_data[i].price);
				PQclear (res);
			}
		}
	}

	if (!is_pickup && the_store == current_store && the_customer)
	{
		// update the customer details, if we are in the same store

		res = execSQL ("update customers set lastorder_num = %d, lastorder_date = 'now', "
					"comments = '%s' "
					"where custnum = %d",
					the_order, 
					SQL(gtk_editable_get_chars (comments,0,-1)),
					the_customer);
		PQclear (res);
	}

	order_entry_quit_cb ();
}

static void order_entry_accept_cb ()
{
	char *generated_docket;

	order_entry_adp_cb ();

	print_docket (the_store, the_order);
}

static void order_entry_cancel_cb ()
{
	if (!is_new && the_store == current_store)
	{
		// delete the old ordet
		res = execSQL ("delete from ordet where store_id = %d and ordernum = %d",
						the_store, the_order);  
		PQclear (res);
		res = execSQL ("delete from orders where store_id = %d and ordernum = %d",
						the_store, the_order);  
		PQclear (res);
		inform_all ();
	}

	order_entry_quit_cb ();
}

static void add_discount_line (double value)
{
	int i;

	for (i = NUM_ROWSS - 1; i >= 0; i--)
	{
		if (strlen(row_data[i].code))
		{
			i++;	// this is the row we will add
			clear_bubble ();
			display_page = i / NUM_ROWS;
			show_bubble ();

			do_change_code = 0;
			// now add the line
			strcpy (row_data[i].code, "DIS");
			strcpy (row_data[i].desc, "Discount");
			row_data[i].qty = 1;
			row_data[i].price = value;
			row_data[i].change_price = 1;
			row_data[i].unit_price = value;
			row_data[i].allow_discount = 0;

			row_data_to_screen ();
			calc_total_price ();
			gtk_widget_grab_focus (GTK_WIDGET (code [i % NUM_ROWS]));

			do_change_code = 1;
			break;
		}
	}
}

gint order_entry_key_press (GtkWidget *widget, GdkEventKey *event)
{
	gint return_val;
	int i;
	char command[255];

	switch (event->keyval)
	{
		case GDK_F1:
			reload_image (pix, "Menu1.jpg");
			current_pix = "Menu1.jpg";
			break;

		case GDK_F2:
			reload_image (pix, "Menu2.jpg");
			current_pix = "Menu2.jpg";
			break;

		case GDK_F3:
			reload_image (pix, "Menu3.jpg");
			current_pix = "Menu3.jpg";
			break;

		case GDK_F4:
			reload_image (pix, "Menu4.jpg");
			current_pix = "Menu4.jpg";
			break;

		case GDK_F5:
			reload_image (pix, "Menu5.jpg");
			current_pix = "Menu5.jpg";
			break;

		case GDK_F6:
			reload_image (pix, "Menu6.jpg");
			current_pix = "Menu6.jpg";
			break;

		case GDK_F7:
			reload_image (pix, "Menu7.jpg");
			current_pix = "Menu7.jpg";
			break;

		case GDK_F8:
			reload_image (pix, "Menu8.jpg");
			current_pix = "Menu8.jpg";
			break;

		case GDK_F9:
			reload_image (pix, "Menu9.jpg");
			current_pix = "Menu9.jpg";
			break;

		case GDK_F10:
			reload_image (pix, "Menu10.jpg");
			current_pix = "Menu10.jpg";
			break;

		case GDK_F12:
			sprintf (command, "xpaint %s &", current_pix);
			system (command);
			break;

		case GDK_Home:
			if (display_page > 0)
			{
				clear_bubble ();
				display_page = 0;
				show_bubble ();
				row_data_to_screen ();
			}
			gtk_widget_grab_focus (GTK_WIDGET(code[0]));
			break;

		case 't':
			if (event->state & 0x04)
				set_total_price ();
			break;

		case GDK_End:
			// scan backward to find the last one
			for (i = NUM_ROWSS - 1; i >= 0; i--)
			{
				if (strlen(row_data[i].code))
				{
					clear_bubble ();
					display_page = i / NUM_ROWS;
					show_bubble ();
					row_data_to_screen ();
					break;
				}
			}
			gtk_widget_grab_focus (GTK_WIDGET(code[i]));
			break;
			
		case GDK_Page_Up:
			if (display_page > 0)
			{
				clear_bubble ();
				display_page--;
				show_bubble ();
				row_data_to_screen ();
			}
			gtk_widget_grab_focus (GTK_WIDGET(code[0]));
			break;
			
		case GDK_Page_Down:
			if (display_page < 9)
			{
				clear_bubble ();
				display_page++;
				show_bubble ();
				row_data_to_screen ();
			}
			gtk_widget_grab_focus (GTK_WIDGET(code[0]));
			break;
	}

	return return_val;
}

static void pix_button_press (GtkWidget *widget, gint row, gint col,
    GdkEventButton *event, gpointer data)
{
	char command[255];

	printf ("pressed btn on pixmap %s\n", current_pix);

	if (event->button != 1)
	{
		sprintf (command, "xpaint %s &", current_pix);
		system (command);
	}
	// else popup a selection list of the valid items
}

static void default_focus ()
{
	gtk_widget_grab_focus (is_pickup ? GTK_WIDGET(pick_name) : GTK_WIDGET(code[0]));
}

static void populate_labels ()
{
	char command[255];
	char Buffer[1024];
	char *ptr;

	apply_discount = 1.0;
	gtk_editable_delete_text (GTK_EDITABLE(comments),0,-1);

	if (is_pickup)
	{
		gtk_label_set (l1, "Pickup By:");
		gtk_label_set (l2, "Time:");

		gtk_widget_hide (GTK_WIDGET(cust_label));
		gtk_widget_hide (GTK_WIDGET(deliv_label));
		gtk_widget_show (GTK_WIDGET(pick_name));
		gtk_widget_show (GTK_WIDGET(pick_time));

		if (is_new)
		{
			gtk_input_set_text (pick_name, "");
			gtk_input_set_text (pick_time, "");
		}
		else
		{
			res = execSQL ("select pickup_name, pickup_time, comments from orders "
					"where ordernum = %d and store_id = %d",
					the_order, the_store);
			gtk_input_set_text (pick_name, PQgetvalue (res,0,0));
			gtk_input_set_text (pick_time, PQgetvalue (res,0,1));
			ptr = trim (PQgetvalue (res,0,2));
        	gtk_text_insert (comments, NULL,NULL,NULL,ptr,strlen(ptr));
			PQclear (res);
		}

		default_focus ();
	}
	else // is delivery
	{
		gtk_label_set (l1, "Cust:");
		gtk_label_set (l2, "Deliv:");

		gtk_widget_hide (GTK_WIDGET(pick_name));
		gtk_widget_hide (GTK_WIDGET(pick_time));
		gtk_widget_show (GTK_WIDGET(cust_label));
		gtk_widget_show (GTK_WIDGET(deliv_label));

		if (is_new)
		{
			// new order, but we know the customer number

			res = execSQL ("select custnum,custname,address,phone,suburb,ubd,discount,"
					"comments,deliv_charge "
					"from customers where "
					"custnum = %d",
					the_customer);
		}
		else
		{
			res = execSQL ("select o.custnum,c.custname,c.address,c.phone,c.suburb,c.ubd,c.discount,"
					"o.comments,c.deliv_charge "
					"from orders o,customers c where "
					"o.ordernum = %d and o.store_id = %d and c.custnum = o.custnum",
					the_order, the_store);
		}
	
		the_customer = atoi(PQgetvalue (res,0,0)); // set the global customer ID

		sprintf (Buffer, "%s, %s", PQgetvalue (res,0,1), PQgetvalue (res,0,3));
		gtk_label_set (cust_label, Buffer);
		sprintf (Buffer, "%s, %s, %s", PQgetvalue (res,0,2), PQgetvalue (res,0,4), PQgetvalue (res,0,5));
		gtk_label_set (deliv_label, Buffer);

		ptr = trim (PQgetvalue (res,0,7));
        gtk_text_insert (comments, NULL,NULL,NULL,ptr,strlen(ptr));

		apply_discount = 1.0 - (atoi(PQgetvalue (res,0,6)) / 100.0);

		cust_deliv_charge = atof(PQgetvalue (res,0,8));

		PQclear (res);
	}

	printf ("Discount of %2.2f applied\n", apply_discount);

	// finally, fill in the totals field
	res = execSQL ("select sum(price) from ordet "
					"where ordernum = %d and store_id = %d",
					the_order, the_store);
	total_price_value = atof(PQgetvalue (res,0,0));
	gtk_label_set (total_price, money (total_price_value, MLEN));
	printf ("Set label to '%s'\n", PQgetvalue (res,0,0));
	PQclear (res);

	default_focus ();
}

static void clear_order_fields ()
{
	int i;
	do_change_code = 0;
	
	for (i = 0; i < NUM_ROWS; i++)
	{
		gtk_input_set_text (code[i], "");
		gtk_input_set_text (desc[i], "");
		gtk_input_set_text (qty[i], "");
		gtk_input_set_text (price[i], "");
		gtk_input_set_editable (price[i], 0);
	}

	do_change_code = 1;
}

static void populate_order_fields ()
{
	int i, nTuples;
	char command[255];

	clear_order_fields ();

	if (is_new) return;

	res = execSQL ("select o.item_code, o.description, o.qty, o.price, o.seq_num, "
				"i.change_price, i.price, c.allow_discount, i.category "
				"from ordet o, items i, category c "
				"where i.code = o.item_code and o.ordernum = %d and o.store_id = %d "
				"and c.code = i.category order by o.seq_num",
				the_order, the_store);
	nTuples = PQntuples (res);

	do_change_code = 0;
	zap_row_data ();

	for (i = 0; i < nTuples && i < NUM_ROWSS; i++)
	{
		strcpy (row_data[i].code, PQgetvalue (res,i,0));
		strcpy (row_data[i].desc, PQgetvalue (res,i,1));
		row_data[i].qty = atoi (PQgetvalue (res,i,2));
		row_data[i].price = atof (PQgetvalue (res,i,3));
		row_data[i].change_price = (*PQgetvalue (res,i,5) == 't');
		row_data[i].unit_price = atof (PQgetvalue (res,i,6));
		row_data[i].allow_discount = (*PQgetvalue (res,i,7) == 't');
	}

	clear_bubble ();
	display_page = 0;	
	show_bubble ();
	row_data_to_screen ();

	do_change_code = 1;
	PQclear (res);
}

static void qty_changed (GtkWidget *widget, int data)
{
	GtkInput *qty;
	int the_qty;
	int i,j;
	float the_price;

	if (!do_change_code) return;

	qty = GTK_INPUT(widget);
	i = (int) data;
	j = ABS_ROW(i);
	the_qty = atoi(gtk_input_get_text (qty));

	if (row_data[j].allow_discount)
		the_price = discounted (the_qty * row_data[j].unit_price);
	else
		the_price = the_qty * row_data[j].unit_price;

	gtk_input_set_text (price[i], money (the_price, MLEN));

	row_data[j].qty = the_qty;
	row_data[j].price = the_price;
	calc_total_price ();
}

static void qty_focus_in (GtkWidget *widget, GdkEventFocus *event, int i)
{
	gtk_input_select_region (qty[i],0,-1);
}

static void qty_focus_out (GtkWidget *widget, GdkEventFocus *event, int i)
{
	gtk_input_select_region (qty[i],0,0);
}

static void desc_changed (GtkWidget *widget, int data)
{
	char *ptr;

	if (!do_change_code) return;

	ptr = gtk_input_get_full_text (GTK_INPUT(widget));
	strcpy (row_data[ABS_ROW(data)].desc, ptr);
}

static void calc_total_price ()
{
	int i;
	total_price_value = 0.0;

	for (i = 0; i < NUM_ROWSS; i++)
		total_price_value += row_data[i].price;

	gtk_label_set (total_price, money (total_price_value, MLEN));
}

static void price_changed (GtkWidget *widget, int data)
{
	float the_price;

	if (!do_change_code) return;

	the_price = atof(gtk_input_get_text (price[data]));
	row_data[ABS_ROW(data)].price = the_price;

	calc_total_price ();
}

static void price_focus_in (GtkWidget *widget, GdkEventFocus *event, int i)
{
	if (row_data[ABS_ROW(i)].change_price)
		gtk_input_select_region (price[i],0,-1);
}

static void price_focus_out (GtkWidget *widget, GdkEventFocus *event, int i)
{
	int old_change_code = do_change_code;
	do_change_code = 0;

	gtk_input_select_region (price[i],0,0);
	gtk_input_set_text (price[i], money (gtk_input_get_double (price[i]), MLEN));

	do_change_code = old_change_code;
}

static void code_focus_in (GtkWidget *widget, GdkEventFocus *event, int i)
{
	gtk_input_select_region (code[i],0,-1);
}

static void code_focus_out (GtkWidget *widget, GdkEventFocus *event, int i)
{
	gtk_input_select_region (code[i],0,0);
}

static void code_changed (GtkWidget *widget, int data)
{
	GtkInput *code;
	char *ptr;
	int i,j,nTuples;
	static char command[256];
	float value;

	if (!do_change_code) return;

	do_change_code = 0;

	code = GTK_INPUT(widget);
	i = (int) data;
	j = ABS_ROW(i);
	ptr = gtk_input_get_text (code);
	//printf ("CODE = '%s'\n", ptr);

	if (strlen (ptr))
	{
		res = execSQL ("select i.description,i.price,i.change_price,i.category,c.allow_discount "
						"from items i, category c where i.code = '%s' and c.code = i.category",
			ptr);
		if (res)
		{
			nTuples = PQntuples (res);
			printf ("Tuples = %d\n", nTuples);
			if (nTuples == 1)
			{
				row_data[j].unit_price = atof(PQgetvalue (res,0,1));
				row_data[j].change_price = (*PQgetvalue (res,0,2) == 't');
				row_data[j].allow_discount = (*PQgetvalue (res,0,4) == 't');

				//if (!strlen(gtk_input_get_text(desc[i])))
				//{
					gtk_input_set_text (desc[i], PQgetvalue (res,0,0));
					gtk_input_set_text (qty[i], "1");
					value = atof (PQgetvalue (res,0,1));
					if (row_data[j].allow_discount)
						value = discounted (value);
					gtk_input_set_text (price[i], money (value, MLEN));
				//}
				gtk_input_set_editable (price[i], *PQgetvalue (res,0,2) == 't');
			}
			else
			{
				gtk_input_set_text (desc[i], "");
				gtk_input_set_text (qty[i], "");
				row_data[j].unit_price = 0.0;
				row_data[j].allow_discount = 0;
				row_data[j].change_price = 0;
				gtk_input_set_text (price[i], "");
			}
			PQclear (res);
		}
		else
		{
			gtk_input_set_text (desc[i], "");
			gtk_input_set_text (qty[i], "");
			gtk_input_set_text (price[i], "");
		}
	}
	else // cleared
	{
		gtk_input_set_text (desc[i], "");
		gtk_input_set_text (qty[i], "");
		gtk_input_set_text (price[i], "");
	}
	gtk_widget_grab_focus (GTK_WIDGET(code));

	row_to_row_data (i);

	do_change_code = 1;
}

/*-------------------------------------------------
   Dialog and global vars for handling total price 
---------------------------------------------------*/

enum {
	TP_PRICE,
	TP_ACCEPT,
	TP_QUIT
};

static GtkInput *tp_price;
static DataEntry *tp = NULL;

static void tp_accept ()
{
	double new_price, difference;

	new_price = gtk_input_get_double (tp_price);
	difference = new_price - total_price_value;
	add_discount_line (difference);
	gtk_widget_hide (GTK_WIDGET(tp));
}

static void tp_quit ()
{
	gtk_widget_hide (GTK_WIDGET(tp));
}

static void set_total_price ()
{
	if (!tp)
	{
		tp = DATA_ENTRY(data_entry_new ("Set Total Price", 100,100,0,0));
		gtk_widget_realize (GTK_WIDGET(tp));

		data_entry_add_label_button (tp, TP_ACCEPT, "Accept", 
				GTK_SIGNAL_FUNC(tp_accept), BUTTON_DEFAULT);
		data_entry_add_label_button (tp, TP_QUIT, "Quit", 
				GTK_SIGNAL_FUNC(tp_quit), BUTTON_QUIT);

		tp_price = GTK_INPUT (data_entry_add_field (tp, TP_PRICE, "New Total Price:", 
			8, INPUT_UPPER, 2,1,1,1));
	}

	if (!GTK_WIDGET_VISIBLE(tp))
		gtk_widget_show (GTK_WIDGET(tp));

	gtk_input_set_text (tp_price, money (total_price_value, MLEN));
	gtk_widget_grab_focus (GTK_WIDGET (tp_price));
	gdk_window_raise (GTK_WIDGET(tp)->window);
}

static void cust_det_cb ()
{
	do_cust_details_from_order (the_customer);
}

/*-------------------------------------------------
   End of tp code
---------------------------------------------------*/

static void order_entry (int pickup)
{
	DataEntry *data_entry;
	GtkWidget *window, *vbox, *comment_label;
	static GtkWidget *history_btn, *cust_det, *future_btn;
	int i;

	is_pickup = pickup;

	if (!order_entry_data_entry)
	{
		order_entry_data_entry = DATA_ENTRY(data_entry_new ("Order Details", 3, 3, 0,0));
		data_entry = order_entry_data_entry;

		window = GTK_WIDGET (data_entry);
		gtk_widget_realize (window);

		gtk_signal_connect (GTK_OBJECT (window), "key_press_event",
			GTK_SIGNAL_FUNC(order_entry_key_press), data_entry);

		history_btn = data_entry_add_label_button (data_entry, ORDER_ENTRY_HISTORY_BTN, "History", 
				GTK_SIGNAL_FUNC(order_entry_history_cb), BUTTON_NORMAL);
		data_entry_add_label_button (data_entry, ORDER_ENTRY_ACCEPT_BTN, "Accept", 
				GTK_SIGNAL_FUNC(order_entry_accept_cb), BUTTON_DEFAULT);
		data_entry_add_label_button (data_entry, ORDER_ENTRY_ADP_BTN, "Accept - Dont Print", 
				GTK_SIGNAL_FUNC(order_entry_adp_cb), BUTTON_NORMAL);
		data_entry_add_label_button (data_entry, ORDER_ENTRY_CANCEL_BTN, "Cancel Order", 
				GTK_SIGNAL_FUNC(order_entry_cancel_cb), BUTTON_NORMAL);
		data_entry_add_label_button (data_entry, ORDER_ENTRY_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(order_entry_quit_cb), BUTTON_QUIT);

		// add the pix

		vbox = data_entry_add_widget (data_entry, -1, gtk_vbox_new (FALSE,5), 10,0,3,18);
    	pix = new_image ("Menu1.jpg", window->window, &window->style->bg[GTK_STATE_NORMAL]);
		gtk_container_add (GTK_CONTAINER(vbox), pix);
    	pixmenu = new_image ("Menu_Keys.jpg", window->window, &window->style->bg[GTK_STATE_NORMAL]);
		gtk_container_add (GTK_CONTAINER(vbox), pixmenu);

		comment_label = gtk_label_new ("Comments:");
		gtk_container_add (GTK_CONTAINER(vbox), comment_label);
		comments = gtk_text_new (NULL, NULL);
		gtk_text_set_editable (comments, TRUE);
		gtk_container_add (GTK_CONTAINER(vbox), comments);
		gtk_widget_set_usize (GTK_WIDGET (comments), 310,60);

		gtk_widget_show (pix);
		gtk_widget_show (pixmenu);
		gtk_widget_show (comment_label);
		gtk_widget_show (comments);

		/*
		pix = data_entry_add_image (data_entry, -1, "Menu1.jpg",
			10,0,3,13);
		//gtk_widget_set_usize (GTK_WIDGET(pix), 100,100);
		current_pix = "Menu1.jpg";
		gtk_signal_connect (GTK_OBJECT (pix), "button_press_event",
			GTK_SIGNAL_FUNC(pix_button_press), data_entry);
		pixmenu = data_entry_add_image (data_entry, -1, "Menu_Keys.jpg",
			10,13,3,3);
		*/

		cust_det = data_entry_add_widget (data_entry, -1, gtk_button_new_with_label ("Cust Det"),
			10,3 + NUM_ROWS + 1,1,1);
		gtk_signal_connect (GTK_OBJECT (cust_det), "clicked",
			GTK_SIGNAL_FUNC(cust_det_cb), data_entry);
		future_btn = data_entry_add_widget (data_entry, -1, gtk_button_new_with_label ("Future Date"),
			11,3 + NUM_ROWS + 1,1,1);

		// add the labels at the top

		data_entry->table_spacing = 2;
		l1 = GTK_LABEL(data_entry_add_widget (data_entry, -1, gtk_label_new (""), 0,0,1,1));
		cust_label = GTK_LABEL(data_entry_add_widget (data_entry, -1, gtk_label_new (""), 1,0,3,1));
		l2 = GTK_LABEL(data_entry_add_widget (data_entry, -1, gtk_label_new (""), 0,1,1,1));
		deliv_label = GTK_LABEL(data_entry_add_widget (data_entry, -1, gtk_label_new (""), 1,1,3,1));

		data_entry_add_widget (data_entry, -1, gtk_label_new ("Code"), 0,2,1,1);
		data_entry_add_widget (data_entry, -1, gtk_label_new ("Description"), 1,2,1,1);
		data_entry_add_widget (data_entry, -1, gtk_label_new ("Qty"), 2,2,1,1);
		data_entry_add_widget (data_entry, -1, gtk_label_new ("Price"), 3,2,1,1);

		// add the pickup fields

		pick_name = GTK_INPUT(data_entry_add_field (data_entry, ORDER_ENTRY_PICKUP_NAME,
			NULL, 40, INPUT_NORMAL, 1,0,4,1));
		pick_time = GTK_INPUT(data_entry_add_field (data_entry, ORDER_ENTRY_PICKUP_TIME,
			NULL, 10, INPUT_UPPER, 1,1,1,1));

		// generate the 12 rows of input fields

		data_entry->table_spacing = 0;
		for (i = 0; i < NUM_ROWS; i++)
		{
			code[i] = GTK_INPUT(data_entry_add_field (data_entry, i*4, NULL, 12, 
				INPUT_UPPER, 0,3+i,1,1));
			gtk_widget_set_usize (GTK_WIDGET(code[i]), 52,24); // shrink the visible size down a bit 5*9
			gtk_signal_connect (GTK_WIDGET(code[i]), "changed",
				GTK_SIGNAL_FUNC(code_changed),i);
			gtk_signal_connect (GTK_WIDGET(code[i]), "focus_in_event",
				GTK_SIGNAL_FUNC(code_focus_in),i);
			gtk_signal_connect (GTK_WIDGET(code[i]), "focus_out_event",
				GTK_SIGNAL_FUNC(code_focus_out),i);

			desc[i] = GTK_INPUT(data_entry_add_field (data_entry, i*4 + 1, NULL, 256, 
				INPUT_NORMAL, 1,3+i,1,1));
			gtk_widget_set_usize (GTK_WIDGET(desc[i]), 280,24); // shrink the visible size down a bit
			gtk_signal_connect (GTK_WIDGET(desc[i]), "changed",
				GTK_SIGNAL_FUNC(desc_changed),i);

			qty[i] = GTK_INPUT(data_entry_add_field (data_entry, i*4 + 2, NULL, 3, 
				INPUT_UPPER, 2,3+i,1,1));
			gtk_signal_connect (GTK_WIDGET(qty[i]), "changed",
				GTK_SIGNAL_FUNC(qty_changed),i);
			gtk_signal_connect (GTK_WIDGET(qty[i]), "focus_in_event",
				GTK_SIGNAL_FUNC(qty_focus_in),i);
			gtk_signal_connect (GTK_WIDGET(qty[i]), "focus_out_event",
				GTK_SIGNAL_FUNC(qty_focus_out),i);

			price[i] = GTK_INPUT(data_entry_add_field (data_entry, i*4 + 3, NULL, MLEN, 
				INPUT_UPPER, 3,3+i,1,1));
			gtk_signal_connect (GTK_WIDGET(price[i]), "changed",
				GTK_SIGNAL_FUNC(price_changed),i);
			gtk_signal_connect (GTK_WIDGET(price[i]), "focus_in_event",
				GTK_SIGNAL_FUNC(price_focus_in),i);
			gtk_signal_connect (GTK_WIDGET(price[i]), "focus_out_event",
				GTK_SIGNAL_FUNC(price_focus_out),i);

			if (i < 10)
				bubble[i] = data_entry_add_image (data_entry, -1, "sphere1.jpg",
					4,3+i,1,1);
		}

		// add the total fields

		total_btn = data_entry_add_widget (data_entry, -1, gtk_button_new_with_label ("Total"),
			1,3 + NUM_ROWS + 1, 2,1);
		gtk_signal_connect (GTK_WIDGET(total_btn), "clicked",
			GTK_SIGNAL_FUNC(set_total_price),NULL);

		total_price = data_entry_add_widget (data_entry, -1, gtk_label_new ("0.00"),
			3,3 + NUM_ROWS + 1, 1,1);

	}

	if (!GTK_WIDGET_VISIBLE(order_entry_data_entry))
		gtk_widget_show (GTK_WIDGET(order_entry_data_entry));

	populate_labels ();

	zap_row_data ();

	if (is_new)
	{
		clear_order_fields ();
	}
	else
	{
		populate_order_fields ();
	}

	if (is_pickup)
	{
		gtk_widget_hide (cust_det);
	}
	else
	{
		gtk_widget_show (cust_det);

		if (is_new && cust_deliv_charge != 0.0)
		{
			// add a line for the deliv charge
			do_change_code = 0;
			i = NUM_ROWS-3;
			strcpy (row_data[i].code, "DLV");
			strcpy (row_data[i].desc, "Delivery");
			row_data[i].qty = 1;
			row_data[i].price = cust_deliv_charge;
			row_data[i].change_price = 1;
			row_data[i].unit_price = cust_deliv_charge;
			row_data[i].allow_discount = 0;

			row_data_to_screen ();
			calc_total_price ();
			do_change_code = 1;
		}
	}

	if (check_if_has_history ())
		gtk_widget_show (history_btn);
	else
		gtk_widget_hide (history_btn);

	default_focus ();

}

void new_order (int pickup, int customer_number)
{
	the_order = 0;
	the_store = current_store;
	is_new = TRUE;
	the_customer = customer_number;

	printf ("new_order (%d, %d)\n", pickup, customer_number);
	order_entry (pickup);
}

void old_order (int store_id, int order_id, int p)
{
	the_store = store_id;
	the_order = order_id;
	is_new = FALSE;

	printf ("old_order (%d, %d, %d)\n", store_id, order_id, p);
	order_entry (p);
}

void order_redisplay ()
{
	gtk_widget_show (GTK_WIDGET(order_entry_data_entry));
	gdk_window_raise (GTK_WIDGET(order_entry_data_entry)->window);
	default_focus ();
}

